<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Memo;
use App\Models\UserEloquent;

class MemoController extends Controller
{

	/**
	 * Show the required memo
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function show( Request $request, int $id )
	{
		return view('memo')
			->with('memo',Memo::findOrFail($id))
			->with('message',$request->session()->get('message'));
	}

	/**
	 * Show the three last memos
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function last( Request $request )
	{
		$lastMemos = Memo::where('public',true)
			->orderBy('published_on','desc')
			->take(3)
			->get();

		return view('home')
			->with('memos',$lastMemos)
			->with('message',$request->session()->get('message'));
	}

	/**
	 * Show the form to write a new memo
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function formMemo( Request $request )
	{
		return view('addmemo')
			->with('message',$request->session()->get('message'));
	}

	/**
	 * Show the form to edit an memo
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @param  int $memo_id
	 * @return \Illuminate\Http\Response
	 */
	public function formeditmemo( Request $request, int $memo_id )
	{
		return view('memoedit')
			->with('memo',Memo::find($memo_id))
			->with('message',$request->session()->get('message'));
	}

	/**
	 * Add a new memo
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function add( Request $request )
	{
		if ( !$request->filled(['title', 'content']) )
			return to_route('view_formmemo')->with('message','Some POST data are missing.');

		$memo = new Memo;
		$memo->owner= $request->session()->get('user')->login;
		$memo->title = $request->title;
		$memo->content = $request->content;
		$memo->public = (bool) $request->public;
		$memo->published_on = $memo->public ? now() : null;
        $memo->last_update = $memo->published_on;

		try
		{
			$memo->save();
		}
		catch (\Illuminate\Database\QueryException $e)
		{
			return to_route('view_formmemo')
				->with('message','Sorry, an error occur durng memo saving. Please try again : ' . $e);
		}

		return to_route('view_account')->with('message','New memo created!');
	}

	/**
	 * Chnage the publication status on/off
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @param  int $memo_id
	 * @return \Illuminate\Http\Response
	 */
	public function toggleStatus( Request $request, int $memo_id )
	{
		$currentDate= now();

		$memo = Memo::find($memo_id);
		if ( $memo->public)
		{
			$memo->public = false;
			$memo->published_on = null;
		}
		else
		{
			$memo->public = true;
			$memo->published_on = $currentDate;
		}

		try
		{
			$memo->save();
		}
		catch (\Illuminate\Database\QueryException $e)
		{
			return to_route('view_memo_list')
				->with('message','Sorry, an error occur durng memo update. Please try again.');
		}

		return to_route('view_memo_list');
	}

	/**
	 * Edit an memo
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @param  int $memo_id
	 * @return \Illuminate\Http\Response
	 */
	public function edit( Request $request, int $memo_id )
	{
		if ( !$request->filled(['title','content']) )
			return to_route('view_memo_list')->with('message','Some POST data are missing.');

		$memo = Memo::find($memo_id);
		$memo->title = $request->title;
		$memo->content = $request->content;
		$memo->last_update = now();

		try
		{
			$memo->save();
		}
		catch (\Illuminate\Database\QueryException $e)
		{
			return to_route('view_memo_list')
				->with('message','Sorry, an error occur during memo saving. Please try again.');
		}

		return to_route('view_memo_list')->with('message','Memo updated!');
	}
}