<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use App\Models\MyUser;

class UserController extends Controller
{
	/**
	 * Signout
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function disconnect( Request $request )
	{
		$request->session()->flush();
		return to_route('view_home');
	}


	/**
	 * Authentication
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function connect( Request $request )
	{
		if ( !$request->filled(['login','password']) )
			return to_route('view_signin')->with('message','Some POST data are missing.');

		try
		{
			$user = MyUser::where('login',$request->login)->firstOrFail();
		}
		catch (\Illuminate\Database\Eloquent\ModelNotFoundException $e)
		{
			return to_route('view_signin')->with('message','Wrong login.');
		}

		if ( !Hash::check($request->password,$user->password) )
			return to_route('view_signin')->with('message','Wrong password.');

		$request->session()->put('user',$user);
		return to_route('view_account');
	}


	/**
	 * Add a user
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function create( Request $request )
	{
		if ( !$request->filled(['login','password','confirm']) )
			return to_route('view_signup')->with('message','Some POST data are missing.');

		if ( $request->password !== $request->confirm )
			return to_route('view_signup')->with('message','Error: the two passwords differ. Try again.');

		$user = new MyUser;
		$user->login = $request->login;
		$user->password = Hash::make($request->password);

		try
		{
			$user->save();
		}
		catch (\Illuminate\Database\QueryException $e)
		{
			return to_route('view_signup')
				->with('message','Sorry, this login still exists. Please choose another one.');
		}

		return to_route('view_signin')->with('message','Account created! Now, signin.');
	}

	/**
	 * Change the user password
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function updatePassword( Request $request )
	{
		if ( !$request->filled(['newpassword','confirmpassword']) )
			return to_route('view_formpassword')->with('message','Some POST data are missing.');

		if ( $request->newpassword != $request->confirmpassword )
			return to_route('view_formpassword')->with('message','Error: the two passwords differ. Try again.');

		try
		{
			$user = $request->session()->get('user');
			$user->password = Hash::make($request->newpassword);
			$user->save();
		}
		catch (\Illuminate\Database\QueryException $e)
		{
			return to_route('view_formpassword')->with('message','Error: password changing failed. Try again.');
		}

		return to_route('view_account')->with('message','Password successfully updated.');
	}

	/**
	 * Delete the logged user
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function delete( Request $request )
	{
		try
		{
			$request->session()->get('user')->delete();
		}
		catch (\Illuminate\Database\Eloquent\ModelNotFoundException $e)
		{
			return to_route('view_account')->with('message','Error during deletion. Try again.');
		}

		$request->session()->flush();
		return to_route('view_signin')->with('message','Account successfully deleted.');
	}
}
