<?php

use Illuminate\Support\Facades\Route;
use App\Http\Controllers\UserController;
use App\Http\Controllers\MemoController;
use App\Http\Middleware\AuthenticateMyUser;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

/* ------------------------------------------------------------------------
| Public routing */
Route::get('/', [MemoController::class,'last'])->name('view_home');

Route::view('/signin','signin')->name('view_signin');
Route::post('/authenticate', [UserController::class, 'connect'])->name('user_authenticate');

Route::view('/signup', 'signup')->name('view_signup');
Route::post('/adduser', [UserController::class, 'create'])->name('user_adduser');
Route::get('memo_show/{memo_id}', [MemoController::class,'show'])->name('view_memo');

// ------------------------------------------------------------------------

/* ------------------------------------------------------------------------
| Admin routing */
Route::prefix('admin')->middleware(AuthenticateMyUser::class)->group(function () {

	Route::view('/account', 'account')->name('view_account');

	Route::view('/formpassword','formpassword')->name('view_formpassword');
	Route::post('/changepassword', [UserController::class, 'updatePassword'])->name('user_changepassword');
	Route::get('/deleteuser', [UserController::class, 'delete'])->name('user_deleteuser');
	Route::get('/signout', [UserController::class, 'disconnect'])->name('user_signout');

	Route::prefix('memo')->group(function () {

		Route::view('/addmemo', 'formmemo')->name('view_formmemo');
		Route::post('/addmemo', [MemoController::class,'add'])->name('memo_addmemo');
		Route::view('/list', 'memos')->name('view_memo_list');
		

		Route::get('/changestatus/{memo_id}', [MemoController::class,'toggleStatus'])->name('memo_changestatus');
		Route::get('/formeditmemo/{memo_id}', [MemoController::class,'formeditmemo'])->name('view_formeditmemo');
		Route::post('/editmemo/{memo_id}', [MemoController::class,'edit'])->name('memo_editmemo');
	});	
});
// ------------------------------------------------------------------------